; Copyright (c) 2004-2010, Laurent Jorda
;   All rights reserves

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; NAME:
;       EXTRACT_IMAGE_GEOMETRY_NAC
;
; PROJECT:
;       OSIRIS
;
; PURPOSE:
;       This routine loads the header of a PDS (or FITS) file and extract the
;       information on the image geometry: image size, coordinates of the
;       window, binning factors. The result is returned by the function in a
;       structure.
;
; CALLING SEQUENCE:
;       g = EXTRACT_IMAGE_GEOMETRY_NAC(fname, FITS=fits, DEBUG=debug, $
;                                     IFAIL=ifail, LOS=los_st)
;
; INPUTS:
;       fname
;         Name of the PDS (or FITS) image whose header contains the
;         information to be extracted and stored into the structure (see
;         OUTPUTS for details).
;
;         The following keywords are used (and must be present in the image
;         header):
;           -------------------------------------------------------------------
;           FITS keyword  PDS keywords            Used as
;           ------------  -----------------       -----------------------------
;           I_COLU1       FIRST_LINE_SAMPLE       X-coordinate of first pixel
;           I_LINE1       FIRST_LINE              Y-coordinate of first pixel
;           I_LSAMP       LINE_SAMPLES            Size of binned window / X
;           I_LINES       LINES                   Size of binned window / Y
;           F_BINW        PIXEL_AVERAGING_WIDTH   Binning factor / X
;           F_BINH        PIXEL_AVERAGING_HEIGHT  Binning factor / Y
;           -------------------------------------------------------------------
;
; OUTPUTS:
;       The function returns the following:
;         Structure containing the following fields:
;
;           xstart: X-coordinate of lower left  unbinned pixel
;           ystart: Y-coordinate of lower left  unbinned pixel
;           xend:   X-coordinate of upper right unbinned pixel (NOT USED)
;           yend:   Y-coordinate of upper right unbinned pixel (NOT USED)
;           nx:     size/X of binned image (pixels)
;           ny:     size/Y of binned image (pixels)
;           binx:   binning factor/X (pixels)
;           biny:   binning factor/Y (pixels)
;
;       WARNING!
;       Reference system is RCCD = first pixel has coordinates (1,1).
;
;       los_st
;         Equatorial coordinates describing the line-of-sight given by the star
;         trackers. The structure contains the following fields, all the
;         angles being in radians:
;           ra   -> right ascension
;           dec  -> declination
;           roll -> roll angle
;         Their counterparts in degrees are also returned in the structure:
;           ra_deg   -> right ascension
;           dec_deg  -> declination
;           roll_deg -> roll angle
;         The roll angle The roll angle is here the angle between the CCD top
;         direction (up) and the direction of the equatorial North (the angle
;         is positive counterclockwise in the image frame). In the image frame
;         for level 1 and level 2 data, West is right when North is up.
;         **THIS NEEDS TO BE CHECKED FOR THE WAC BECAUSE THERE IS A FLIP**
;
; KEYWORD PARAMETERS:
;       fits
;         Set this keyword if you want the function to extract the information
;         from a FITS header. In this case the image 'fname' must be in FITS
;         format. By default, if this keyword is not set, the information is
;         extracted from a PDS image.
;
;       debug
;         Set to select debug mode. Optional.
;
;       ifail
;         Return the integer error code: 0=OK, >0=error occured
;
; DEPENDENCIES:
;       HEADFITS(ASTRON).
;       SXPAR(ASTRON).
;
; REMARKS:
;       Requires the ASTRON and PDSLIB libraries.
;
; MODIFICATION HISTORY:
;       2006/02/27: written by L. Jorda (LAM)
;       2006/11/28: information extracted also from PDS header (LJ)
;       2007/02/09: computation of the LOS information (LJ)
;       2008/09/09: Changed to fit conventions of calibration pipeline (SFH)
;------------------------------------------------------------------------------
FUNCTION EXTRACT_IMAGE_GEOMETRY_nac, header, DEBUG=debug, IFAIL=ifail, LOS=los_st
    ;
    ; initialization
    ;
    debug = keyword_set(debug)
    ifail = 0
    ct    = 180.0d0/!dpi
    atol  = 1.0d0/3600.0d0/ct   ; 1 arcsec in radians

    if (debug) then print,'M: EXTRACT_IMAGE_GEOMETRY_01 - BEGIN'

    ;
    ; EXTRACT INFORMATION FROM PDS HEADER
    ; -----------------------------------
    if (debug) then print,'M:Extracting information from PDS header.'
    ;
    ; read PDS header and check status
    ;

    ;
    ; extract binned image size from header
    ;
    nx = p_value(header,'IMAGE.LINE_SAMPLES',u)        ; PDS "LINE_SAMPLES"
    ny = p_value(header,'IMAGE.LINES',u)               ; PDS "LINES"

    if (debug) then print,'M: -- Image Size: ',nx,'x',ny,' pixels.'

    ;
    ; extract coordinates of unbinned pixels from header
    ;
    x1 = p_value(header,'IMAGE.FIRST_LINE_SAMPLE',u)         ; PDS "FIRST_LINE_SAMPLE"
    y1 = p_value(header,'IMAGE.FIRST_LINE',u)                ; PDS "FIRST_LINE"
    x2 = x1+nx-1
    y2 = y1+ny-1

    if (debug) then print,'M: -- Image window: [',x1,',',y1,':',x2,',',y2,'].'

    ;
    ; extract binning factor from header
    ;
    bstr = p_value(header,'SR_ACQUIRE_OPTIONS.HARDWARE_BINNING_ID',u)
    barr = fix(strsplit(bstr,'x',/extract))
    binx = barr[0]
    biny = barr[1]

    if (debug) then print,'M: -- Binning factor: ',binx,'x',biny,'.'

    ;
    ; extract RA, DEC and ROLL from header
    ;
    alf0 = p_value(header,'RIGHT_ASCENSION')
    del0 = p_value(header,'DECLINATION')
    rol0 = p_value(header,'NORTH_AZIMUTH')

    if (debug) then begin
        print,'M:Line-of-sight in header.'
        print,'M: -- Right ascension: ',alf0,' deg.'
        print,'M: -- Declination    : ',del0,' deg.'
        print,'M: -- Roll angle     : ',rol0,' deg.'
    endif

    ;
    ; conversion in radians
    ;
    alf0 = alf0/ct
    del0 = del0/ct
    rol0 = rol0/ct

    ;
    ; extract quaternion from header and compute line-of-sight
    ;
    q = double(p_value(header,'SC_COORDINATE_SYSTEM.ORIGIN_ROTATION_QUATERNION',u))

    QUATERNION_TO_EQU, q, alf1, del1, rol1, DEBUG=debug, IFAIL=ifail

    if (ifail ne 0) then begin
        if (debug) then print,'E:Error ',ifail,' calling QUATERNION_TO_EQU_01!'
        ifail = 2
        return, -1
    endif

    if (debug) then begin
        print,'M:Line-of-sight from quaternion.'
        print,'M: -- Right ascension: ',alf1*ct,' deg.'
        print,'M: -- Declination    : ',del1*ct,' deg.'
        print,'M: -- Roll angle     : ',rol1*ct,' deg.'
    endif

    ;
    ; compute final equatorial coordinates
    ;
    alf = alf0      ; by default, we assume that the coordinates in the
    del = del0      ; PDS header are correct ...
    rol = rol0

    if (abs(alf0-alf1) gt atol) then alf = alf1 ; ... but if not we rely on those
    if (abs(del0-del1) gt atol) then del = del1 ; derived from the quaternion
    if (abs(rol0-rol1) gt atol) then rol = rol1

    if (debug) then begin
        print,'M:Line-of-sight /// FINAL VALUES.'
        print,'M: -- Right ascension: ',alf1*ct,' deg.'
        print,'M: -- Declination    : ',del1*ct,' deg.'
        print,'M: -- Roll angle     : ',rol1*ct,' deg.'
    endif

    ;
    ; BUILD STRUCTURE
    ; ---------------
    ; check values
    ;
    if ((binx lt 1) or (biny lt 1) or (nx lt 1) or (ny lt 1) or $
       (x1 lt 1) or (y1 lt 1) or (nx gt 2048) or (ny gt 2048)) then begin
        if (debug) then print,'E:Invalid image and/or incorrect FITS header!'
        ifail = 2
        return,-1
    endif
    ;
    ; create 'geomdef' structure
    ;
    if (debug) then print,'M:Building final structure.'

    geomdef = { $
            xstart: x1, $
            ystart: y1, $
            xend:   x2, $
            yend:   y2, $
            binx:   binx, $
            biny:   biny, $
            nx:     nx, $
            ny:     ny }

    ;
    ; create 'los_st' structure
    ;
    los_st = { $
           ra       : alf, $
           dec      : del, $
           roll     : rol, $
           ra_deg   : alf*ct, $
           dec_deg  : del*ct, $
           roll_deg : rol*ct }

    ;
    ; normal end
    ;
    if (debug) then print,'M: EXTRACT_IMAGE_GEOMETRY_01 - END'

    ; fin
    return, geomdef
END


