; Copyright (c) 2004-2010, Stubbe F. Hviid
;   All rights reserves
;+
; NAME:
;   FUNCTION osiris_level2_to_level3, header_in, image_in, header_out, image_out, quality_map, sigma_map
;
; PURPOSE:
;   Main interface function to the OSIRIS level 2 to level 3 image converter
;   The function will convert from calibrated [W/m^2/sr/nm] unit images to images where the geometric distortion has been removed
;
; CATEGORY:
;   Osiris Geometric Calibration
;
; CALLING SEQUENCE:
;  Result = osiris_level2_to_level3(header_in, image_in, header_out, image_out, quality_map, sigma_map)
;
; INPUTS:
;   header_in:   input level 1 PDS header structure
;   image_in:  input level 1 image data
; 	quality_map		The quality map of the image
;	sigma_map:		The sigma map of the image
;
; KEYWORD PARAMETERS:
;   CODMAC:		  If set the CODMAC data levels are used - else OSIRIS
;
; OUTPUTS:
;   header_out:     output PDS image header
;   image_out:       output calibrated image data
; 	quality_map		The quality map of the image
;	sigma_map:		The sigma map of the image
;
; RETURNS:
;   0 on failure   1 on success
;
; RESTRICTIONS:
;   function requires that the paths defined in get_osiris_environment.pro has been set to correctly
;   point to the root directory of the external calibration files
;
; EXAMPLE:
;   p_read, pickfile(), h_in, d_in                 ; read in raw PDS image
;   osiris_level1_to_level2, h_in, d_in, h_out2, d_out2 ; calibrate the image
;   osiris_level2_to_level3, h_out2, d_out2, h_out3, d_ou3t ; calibrate the image
;   p_write, pickfile(), h_out3, d_out3             ; save the image to a new PDS file
;
; MODIFICATION HISTORY:
;   Written by:  Stubbe F. Hviid   1/3-2007
;-

FUNCTION osiris_level2_to_level3, header_in, image_in, header_out, image_out, quality_map, sigma_map, CODMAC=CODMAC, _EXTRA=_EXTRA
VERSION = '1.0'

    ; initiate status variable
    ok = 1

	  ; do the calibration
    osiris_model, header_in, camera, model

	  if camera eq 'NAC' then begin
        ok = osiris_level2_to_level3_nac(header_in, image_in, header_out, image_out, quality_map, sigma_map)
	  endif else if camera eq 'WAC' then begin
        ok = osiris_level2_to_level3_wac(header_in, image_in, header_out, image_out, quality_map, sigma_map)
	  endif else begin
		    print, 'ERROR:   Unknown Camera - ' + camera
        return, 0
    endelse

    ; update header
	; set the pipeline version to the calibration group
    p_approc,header_out,header_out,'osiris_level2_to_level3','V'+VERSION
    p_setvalue, header_out, 'LEVEL3_PIPELINE.L3_PIPELINE_MASTER_VERSION', VERSION, /GROUP, /HISTORY

    p_setvalue, header_out, 'PRODUCT_TYPE', 'RDR'
    if keyword_set(CODMAC) then begin
    	p_setvalue, header_out, 'PROCESSING_LEVEL_ID', 4	; Setting CODMAC data level 4
    	dsi = p_value(header_out, 'DATA_SET_ID')
    	dsi = strmid(dsi, 0, 12) + '4' + strmid(dsi, 13)
    	p_setvalue, header_out, 'DATA_SET_ID', dsi

		dsn = p_value(header_out, 'DATA_SET_NAME')
		p = strpos(dsn, 'OSI')
		if p gt 0 then dsn = strmid(dsn, 0, p+7) + '4 RDR data'
		p_setvalue, header_out, 'DATA_SET_NAME', dsn
    endif else begin
    	p_setvalue, header_out, 'PROCESSING_LEVEL_ID', 3	; Setting OSIRIS data level 3
    	dsi = p_value(header_out, 'DATA_SET_ID')
    	dsi = strmid(dsi, 0, 5) + 'OSIRIS-3' + strmid(dsi, 13)
    	p_setvalue, header_out, 'DATA_SET_ID', dsi

		dsn = p_value(header_out, 'DATA_SET_NAME')
		p = strpos(dsn, 'OSI')
		if p gt 0 then dsn = strmid(dsn, 0, p) + ' OSIRIS 3 RDR data'
		p_setvalue, header_out, 'DATA_SET_NAME', dsn
	endelse
    p_setvalue, header_out, 'PROCESSING_LEVEL_DESC', 'Geometrically Calibrated Data'
    p_setvalue, header_out, 'PRODUCER_FULL_NAME', strtrim(getenv('USERNAME'), 2)
    p_setvalue, header_out, 'PRODUCT_CREATION_TIME', p_timenow(), /NO_QUOTES, /LABEL

    ; finish
    return, ok
END