; Copyright (c) 2004-2010, Stubbe F. Hviid
;   All rights reserved
;+
; NAME:
;   FUNCTION osiris_abscal, header, image, quality_map, sigma_map
;
; PURPOSE:
;   Convert from DN/s to W/m^2/sr/nm
;
; CATEGORY:
;   Osiris CCD Calibration
;
; CALLING SEQUENCE:
;  Result = osiris_abscal(header, image, quality_map, sigma_map)
;
; INPUTS:
;   header:   		The PDS image header of the image to be corrected
;   image:      	The image data of the image to be corrected
;   quality_map:	Existing map of quality estimate for each pixel to be modified
;   sigma_map:		Existing map of estimaped relative error for each pixel (in %) to be modified
;
; KEYWORD PARAMETERS:
;   PARAMETER_FILELIST:	all external datafile names neede for this correction step
;				        will be added to this keywor parameter in a string array
;
; OUTPUTS:
;   header:   		The PDS image header of the image after correction
;   image:      	The image data of the image after correction
;   quality_map:	output map of quality estimate for each pixel
;   sigma_map:		output map of estimaped relative error for each pixel (in %)
;
; RETURNS:
;   0 on failure   1 on success
;
; RESTRICTIONS:
;   function requires that the paths defined in get_osiris_environment.pro has been set to correctly
;   point to the root directory of the external calibration files
;
;   The function depends on an external data file <camera>_<unit>_AbsCal_<version>_<timestamp>.img
;   to exist in the AbsCal folder of the database tree. This file must be a tag list file containg the
;   conversion constants neede coded in tag of the form  abs<filter #1><filter #2>_A (example abs22_A)
;
; EXAMPLE:
;   p_read, pickfile(), h, d             ; read in raw PDS image
;   osiris_abscal, h, d
;
; MODIFICATION HISTORY:
;   Written by:  Stubbe F. Hviid   28/6-2005
;                Sfh 02102006 - Switched to using osiris_model for detecting the camera and hardware model
;                MK, 26 01 2009: Add line to filter determination to allow reduction of old images
;-


FUNCTION lookup_abscal_param, params, filter, param, TAG=TAG

	tag = strupcase('abs' + strtrim(filter, 2) + '_' + strtrim(param, 2))

	names = strupcase(TAG_NAMES(params))

	s = where(names  eq tag)

	if s[0] eq -1 then begin
		print, 'Cannot find abscal params for: ' + tag
		return, -1
	endif

	TAG = tag

	return, params.(s[0])
END


FUNCTION osiris_abscal, header, image, quality_map, sigma_map, PARAMETER_FILELIST=PARAMETER_FILELIST, _EXTRA=_EXTRA
VERSION='V1.0'

    if keyword_set(VERBOSE) then print, 'osiris_abscal running'

    ; Verify the the correction has not already been performed
    flag = p_value(header, 'LEVEL2_PIPELINE.ROSETTA:ABSOLUTE_CALIBRATION_FLAG', /HISTORY)

    if flag eq 'TRUE' then begin
       if keyword_set(VERBOSE) then print, 'osiris_abscal skipped - Already performed'
       return, 1
    endif

    ; do the calibration

	; extract some information from the image header
	osiris_model, header, camera, model

	fnum = p_value(header, 'SR_MECHANISM_STATUS.FILTER_NUMBER')
    if n_elements(fnum) eq 2 then fnum = string(fnum[0],format='(I1)')+string(fnum[1],format='(I1)')

	; load required flatfield from database
	param_file = p_resolve_database_file('ABSCAL', camera, model, 'ABSCAL', '', '*')

	if param_file eq '' then begin
		print, 'Cannot find required abscal parameter file: ' + param_file + ' - osiris_abscal'
		return, 0
	endif

	; load the parameters
	params = fw_load_tagfile(param_file)

	; add the loaded parameter file to the PARAMETER_FILELIST
	if n_elements(PARAMETER_FILELIST) eq 0 then PARAMETER_FILELIST = [param_file] else PARAMETER_FILELIST = [PARAMETER_FILELIST, param_file]

	; lookup calibration constant
	scale = lookup_abscal_param(params, fnum, 'A', TAG=TAG)

	if scale eq -1 then begin
		printf, -2, 'ERROR: Cannot find abscal tag: ' + TAG + ' - osiris_abscal'
		return, 0
	endif

	; scale the image
	image = image / scale

	p_setvalue, header, 'LEVEL2_PIPELINE.ABSCAL_FACTOR', scale, /HISTORY


	proc_message = 'AbsCal using tag ' + TAG + ' with scaling parameter ' + strtrim(scale, 2)


    ; Change the processing flag
    p_setvalue, header, 'LEVEL2_PIPELINE.ABSOLUTE_CALIBRATION_FLAG', 'TRUE', /LABEL, /NO_QUOTES, /HISTORY
    p_setvalue, header, 'LEVEL2_PIPELINE.ABSOLUTE_CALIBRATION_VERSION', VERSION, /GROUP, /HISTORY


    ; finish
    if keyword_set(VERBOSE) then print, 'osiris_abscal - Done'

    return, 1
END