; Copyright (c) 2004-2010, Stubbe F. Hviid
;   All rights reserved
;+
; NAME:
;   PRO osiris_calibrate_file, in_file, out_file_level2, out_file_level3
;
; PURPOSE:
;   Calibrate a single OSIRIS level1 image file to a level2 (and optionally level3) file
;
; CATEGORY:
;   PRO osiris_calibrate_file(in_file, out_file_level2, out_file_level3)
;
; CALLING SEQUENCE:
;  osiris_calibrate_file, in_file, out_file_level2, out_file_level3
;
; INPUTS:
;   in_file:			The image file to be calibrated
;	out_file_level2:	Where to save the level2 files
;	out_file_level3:	Where to save the level3 files
;
;
; KEYWORD PARAMETERS:
;   REPROCESS=REPROCESS:			Force a reprocessing of all the image (if not set only image that have not yet been processed will be used)
;	DELETE_HISTORY=DELETE_HISTORY:	If set the processing history will be deleted at the end of processing
;	PDS_ARCHIVE=PDS_ARCHIVE:		If set the processing run is for a PDS/PSA archive generation and PDS compatibility will be ensured
;   SKIP_LEVEL3						If set the processing will skip the level three calibration step
;	SKIP_ABS						If set the absolute calibration step is skipped (output will be DN/s)
;	SKIP_EXPTIME					If set the exposure time calibration step is skipped
;	SKIP_FLATFIELD=SKIP_FLATFIELD	If set the flatfielding step is skipped
;	LEVEL2_IMAGE=LEVEL2_IMAGE		Return the level2 image data
;	LEVEL2_HEADER=LEVEL2_HEADER		Return the level2 modified header
;	LEVEL3_IMAGE=LEVEL2_IMAGE		Return the level3 image data
;	LEVEL3_HEADER=LEVEL2_HEADER		Return the level3 modified header
;   OK								Return status:  OK = 1 (success)  OK = 0 (failure)
;
; OUTPUTS:
;   Nothing
;
; RETURNS:
;   Nothing
;
; RESTRICTIONS:
;   function requires that the paths defined in get_osiris_environment.pro has been set to correctly
;   point to the root directory of the external calibration files
;
; EXAMPLE:
;   osiris_calibrate_file, 'level1.img', 'level2.img', 'level3.img'
;
; MODIFICATION HISTORY:
;   Written by:  Stubbe F. Hviid   28/6-2005
;                Sfh 02102006 - Switched to using osiris_model for detecting the camera and hardware model
;-
PRO osiris_calibrate_file, in_file, out_file_level2, out_file_level3, $
						   REPROCESS=REPROCESS, $
                           DELETE_HISTORY=DELETE_HISTORY, $
                           PDS_ARCHIVE=PDS_ARCHIVE, $
                           SKIP_LEVEL3=SKIP_LEVEL3, $
                           SKIP_ABS=SKIP_ABS, $
                           SKIP_EXPTIME=SKIP_EXPTIME, $
                           SKIP_FLATFIELD=SKIP_FLATFIELD, $
                           LEVEL2_IMAGE=LEVEL2_IMAGE, $
                           LEVEL2_HEADER=LEVEL2_HEADER, $
                           LEVEL3_IMAGE=LEVEL3_IMAGE, $
                           LEVEL3_HEADER=LEVEL3_HEADER, $
                           OK=ok, $
                           _EXTRA=_EXTRA


if (N_PARAMS() lt 2) or (N_PARAMS() gt 3) then begin
   printf, -2,'ERROR: osiris_calibrate_file: At least input file and level2 output file need to be specified. Returning.'
   return
endif

ok = 1

if keyword_set(PDS_ARCHIVE) then codmac = 1 else codmac = 0

print, 'Start calibrating: ' + in_file

; generate output filename
fd = p_parse_file_path(in_file)

       ; check compatible filetypes
if strpos(fd.name, 'PA') gt 0 then begin
  print, in_file + ' is a prepixel image - Skipping'
  return
endif

if strpos(fd.name, 'PB') gt 0 then begin
  print, in_file + ' is a prepixel image - Skipping'
  return
endif

if strpos(fd.name, 'OL') gt 0 then begin
  print, in_file + ' is an overclocked line image - Skipping'
  return
endif

if strupcase(strmid(fd.name,0,1)) eq 'N' then camera = 'NAC' else camera = 'WAC'
if camera eq 'NAC' AND strpos(fd.name, '_F54') gt 0 then begin
  print, in_file + ' is a dark/bias image - Skipping'
  return
endif

if camera eq 'WAC' AND strpos(fd.name, '_F87') gt 0 then begin
  print, in_file + ' is a dark/bias image - Skipping'
  return
endif



skip2 = 0
if (NOT keyword_set(REPROCESS)) AND file_test(out_file_level2) ne 0 then begin
  print, out_file_level2 + ' already exists - Skipping'
  skip2 = 1
endif

skip3 = 0
if keyword_set(SKIP_LEVEL3) then begin
   skip3 = 1
endif else begin
   if n_elements(out_file_level3) lt 1 then begin
     print, out_file_level3 + ' not specified - Skipping level 3'
     skip3 = 1
   endif else begin
      if (NOT keyword_set(REPROCESS)) AND file_test(out_file_level3) ne 0 then begin
          print, out_file_level3 + ' already exists - Skipping level 3'
          skip3 = 1
      endif
   endelse
endelse

; if nothing is to be done then simply return
if skip2 eq 1 and skip3 eq 1 then return

; read input file
if skip2 eq 0 then begin
	p_read, in_file, h, d, OK = ok, /QUIET
endif else begin
	p_read, out_file_level2, h2, d2, OK = ok, /QUIET
	quality_map = p_read_image_object(h2, 'QUALITY_MAP_IMAGE')
	sigma_map = p_read_image_object(h2, 'SIGMA_MAP_IMAGE')
endelse
if (ok eq 0) then begin
   printf, -2,'ERROR: reading '+ in_file
   return
endif

; calibrate the file unless told otherwise

if skip2 eq 0 then begin
    ok2 = osiris_level1_to_level2(h, d, h2, d2, quality_map, sigma_map, _EXTRA=_EXTRA, $
                                  SKIP_ABS=SKIP_ABS, SKIP_EXPTIME=SKIP_EXPTIME, $
                                  SKIP_FLATFIELD=SKIP_FLATFIELD, $
                                  CODMAC=codmac)
    if ok2 ne 0 then begin
         ; Delete the revision history if requested (Fixing problems with PDS)
        if keyword_set(DELETE_HISTORY) then begin
             p_delete_tag, h2, 'PROCESSING_HISTORY_TEXT'
        endif

		if keyword_set(PDS_ARCHIVE) then begin
			osiris_make_pds_compatible, h2
		endif

		fs = p_parse_file_path(out_file_level2)
	  	p_setvalue, h2, 'PRODUCT_ID', fs.name

		; save image and header for return to caller
		LEVEL2_IMAGE = d2
        LEVEL2_HEADER = h2

         ; save calibrated image
        p_write, out_file_level2, h2, float(d2), 'QUALITY_MAP_IMAGE', byte(quality_map), 'SIGMA_MAP_IMAGE', float(sigma_map), /VERSION_CONTROL, HEADER_OUT=htmp
         ; change the h2 structure to the modified header
        h2 = htmp

        print, 'Success - Level2'
    endif else begin
        printf, -2, 'ERROR: Failure - Level2'
        return
    endelse
endif

if skip3 eq 0 then begin   ; create level 3 image unless told otherwise
   ok3 = 0
   ok3 = osiris_level2_to_level3(h2, d2, h3, d3, quality_map, sigma_map, _EXTRA=_EXTRA, CODMAC=codmac)

   if ok3 ne 0 then begin
         ; Delete the revision history if requested (Fixing problems with PDS)
      if keyword_set(DELETE_HISTORY) then begin
            p_delete_tag, h3, 'PROCESSING_HISTORY_TEXT'
      endif

	  if keyword_set(PDS_ARCHIVE) then begin
			osiris_make_pds_compatible, h3
	  endif

	  fs = p_parse_file_path(out_file_level3)
	  p_setvalue, h3, 'PRODUCT_ID', fs.name


	  ; save image and header for return to caller
	  LEVEL3_IMAGE = d3
      LEVEL3_HEADER = h3


         ; save calibrated image
      p_write, out_file_level3, h3, float(d3), 'QUALITY_MAP_IMAGE', byte(quality_map), 'SIGMA_MAP_IMAGE', float(sigma_map), /VERSION_CONTROL

      print, 'Success - Level3'
   endif else begin
      printf, -2, 'ERROR: Failure - Level3'
   endelse
endif

end





