; Copyright (c) 2004-2010, Stubbe F. Hviid
;   All rights reserved
;+
; NAME:
;   FUNCTION osiris_darkcurrent_removal, header, image, quality_map, sigma_map
;
; PURPOSE:
;   Remove darkcurrent from an Osiris image
;
; CATEGORY:
;   Osiris CCD Calibration
;
; CALLING SEQUENCE:
;  Result = osiris_darkcurrent_removal(header, image, quality_map, sigma_map)
;
; INPUTS:
;   header:   The PDS image header of the image to be corrected
;   image:      The image data of the image to be corrected
;   quality_map:	Existing map of quality estimate for each pixel to be modified
;   sigma_map:		Existing map of estimaped relative error for each pixel (in %) to be modified
;
; KEYWORD PARAMETERS:
;   PARAMETER_FILELIST: all external datafile names neede for this correction step
;                  will be added to this keywor parameter in a string array
;
; OUTPUTS:
;   header:   The PDS image header of the image after correction
;   image:      The image data of the image after correction
;   quality_map:	output map of quality estimate for each pixel
;   sigma_map:		output map of estimaped relative error for each pixel (in %)
;
; RETURNS:
;   0 on failure   1 on success
;
; RESTRICTIONS:
;   function requires that the paths defined in get_osiris_environment.pro has been set to correctly
;   point to the root directory of the external calibration files
;
;   The input image data is assumed to be in units of DN/s
;
; EXAMPLE:
;   p_read, pickfile(), h, d             ; read in raw PDS image
;   osiris_darkcurrent_removal, h, d
;
; MODIFICATION HISTORY:
;   Written by:  Stubbe F. Hviid   28/6-2005
;                Replaced d_value by p_value at one instance. MK, 22/9-2005
;                Make function work correctly even if exposure time division has not been performed. MK, 05 Nov. 2005
;                Sfh 02102006 - Switched to using osiris_model for detecting the camera and hardware model
;
;-

FUNCTION osiris_darkcurrent_removal, header, image, quality_map, sigma_map, PARAMETER_FILELIST=PARAMETER_FILELIST, _EXTRA=_EXTRA
VERSION='V1.2'

    if keyword_set(VERBOSE) then print, 'osiris_darkcurrent_removal running'

    ; Verify the the correction has not already been performed
    flag = p_value(header, 'LEVEL2_PIPELINE.DARK_CURRENT_CORRECTION_FLAG', /HISTORY)

    if flag eq 'TRUE' then begin
       if keyword_set(VERBOSE) then print, 'osiris_darkcurrent_removal skipped - Already performed'
       return, 1
    endif

    ; do the calibration

    ; read some header information
    osiris_model, header, camera, model

    bin = p_value(header, 'SR_ACQUIRE_OPTIONS.HARDWARE_BINNING_ID')
    swbin = (p_value(header, 'SR_COMPRESSION.PIXEL_AVERAGING_WIDTH'))[0]
    xoff = p_value(header, 'IMAGE.FIRST_LINE_SAMPLE')-1
    yoff = p_value(header, 'IMAGE.FIRST_LINE')-1

    ; first find database file
    param_file = p_resolve_database_file('DARKS', camera, model, 'DARK', '', 'IMG')
    if param_file eq '' then begin
       printf, -2, 'ERROR: Cannot find required parameter file - osiris_darkcurrent_removal'
       return, 0
    endif

    ; add the loaded parameter file to the PARAMETER_FILELIST
    if n_elements(PARAMETER_FILELIST) eq 0 then PARAMETER_FILELIST = [param_file] else PARAMETER_FILELIST = [PARAMETER_FILELIST, param_file]

    ; detemine active binning factor
    bf = 1
    if bin eq '2x2' then bf = 2
    if bin eq '4x4' then bf = 4
    if bin eq '8x8' then bf = 8

    if swbin ne 1 then bf = bf * reform(swbin[0])

    ; do the dark correction
    p_read, param_file, hd, dd

    ; cut out the correct subframe
    s = size(image, /DIM)

    if (xoff + s[0]*bf - 1) ge 2048 then return, 0
    if (yoff + s[1]*bf - 1) ge 2048 then return, 0

    dd = dd[xoff:xoff + s[0]*bf - 1, yoff:yoff + s[1]*bf - 1]
    dd = rebin(dd, s[0], s[1])

    ; scale the dark current by the exposure time
    flag = p_value(header, 'SR_PROCESSING_FLAGS.EXPOSURETIME_CORRECTION_FLAG')
    if flag eq 'FALSE' then dd =  dd * p_value(header, 'SR_ACQUIRE_OPTIONS.EXPOSURE_DURATION')

    ; subtract the darkcurrent from the image
    image = image - dd


    ; post handling
    fd = p_parse_file_path(param_file)
    proc_message = 'osiris_darkcurrent_removal using: ' + fd.filename


    ; Change the processing flag
    p_setvalue, header, 'LEVEL2_PIPELINE.DARK_CURRENT_CORRECTION_FLAG', 'TRUE', /LABEL, /NO_QUOTES, /HISTORY
    p_setvalue, header, 'LEVEL2_PIPELINE.DARK_CURRENT_CORRECTION_VERSION', VERSION, /GROUP, /HISTORY


    ; finish
    if keyword_set(VERBOSE) then print, 'osiris_darkcurrent_removal - Done'

    return, 1
END
