; Copyright (c) 2004-2010, Stubbe F. Hviid
;   All rights reserved
;+
; NAME:
;   FUNCTION osiris_mk_quality_map, header, image, quality_map, sigma_map
;
; PURPOSE:
;   Generate the quality map of the image data
;   bit 0:		set if sqrt filter used
;   bit 1:      set if smoothed
;   bit 2:      set if non lossless compressor used
;
; CATEGORY:
;   Osiris CCD Calibration
;
; CALLING SEQUENCE:
;  Result = osiris_mk_quality_map(header, image, quality_map, sigma_map)
;
; INPUTS:
;   header:   		The PDS image header of the image to be corrected
;   image:     		The image data of the image to be corrected
;   quality_map:	Existing map of quality estimate for each pixel to be modified
;
; OUTPUTS:
;   quality_map:		output map of estimaped relative error for each pixel (in %)
;
; RETURNS:
;   0 on failure   1 on success
;
; RESTRICTIONS:
;   function requires that the paths defined in get_osiris_environment.pro has been set to correctly
;   point to the root directory of the external calibration files
;
;   The function depends on an external data file <camera>_<unit>_BIAS_V<version>_<timestamp>.dat
;   to exist in the BIAS folder of the database tree. This file must have a tag_file compatible structure
;   The file must contain tags with names of the form:
;                                   <frame mode>_<amblifier>_<binning>
;   As a minimum the file must contain the tag "default" giving the default bias level to use if no better
;   value can be found
;
; EXAMPLE:
;   p_read, pickfile(), h, d             ; read in raw PDS image
;   print, osiris_mk_quality_map(h, d, q, s)
;
; MODIFICATION HISTORY:
;   Written by:  Stubbe F. Hviid   4/5-2009
;-

FUNCTION osiris_mk_quality_map, header, image, quality_map
VERSION = '2.0'

	ok = 1

	seg_x = p_value(header, 'SR_COMPRESSION.SEGMENT_X')
	seg_y = p_value(header, 'SR_COMPRESSION.SEGMENT_Y')
	seg_w = p_value(header, 'SR_COMPRESSION.SEGMENT_W')
	seg_h = p_value(header, 'SR_COMPRESSION.SEGMENT_H')

	lf = p_value(header, 'SR_COMPRESSION.LOSSLESS_FLAG')
	cr = p_value(header, 'SR_COMPRESSION.COMPRESSION_RATIO')

	sqf = p_value(header, 'SR_COMPRESSION.SQRT_FILTER_FLAG')
	fid = p_value(header, 'SR_COMPRESSION.SMOOTH_FILTER_ID')

	; get list of valid quality map values
	qmv = osiris_quality_values()

	; handle each segment seperately
	for i=0, n_elements(seg_x)-1 do begin
		x = seg_x[i]
		y = seg_y[i]
		w = seg_w[i]
		h = seg_h[i]
		s = size(image, /dim)

		if x ge 0 AND x lt w AND y ge 0 AND y lt h then begin
			if x+w gt s[0] then w = s[0] - x
			if y+h gt s[1] then h = s[1] - y

			if w gt 0 AND h gt 0 then begin
				sf = image[x:x + w - 1, y:y + h - 1]
				if lf[i] ne 'TRUE' then sf[*] = sf[*] OR qmv.lossy_comp			; mark lossy compression
				if sqf[i] eq 'TRUE' then sf[*] = sf[*] OR qmv.sqrt_filtered		; mark sqrt filtered data
				if fid[i] eq 'TRUE' then sf[*] = sf[*] OR qmv.conv_filtered		; mark convolution filtered data

			    image[x:x + w - 1, y:y + h - 1] = sf
			endif
		endif
	endfor

	; detect pixel non-linearity and saturation
	if ok eq 1 then ok = osiris_detect_saturation(header, image, quality_map)


	; save the function version
	p_setvalue, header, 'LEVEL2_PIPELINE.MK_QUALITY_MAP_VERSION', VERSION, /GROUP, /HISTORY

	return, ok

END